searchData={"items":[{"type":"module","title":"PhxComponentHelpers","doc":"`PhxComponentHelpers` are helper functions meant to be used within Phoenix\nLiveView live_components to make your components more configurable and extensible\nfrom your templates.\n\nIt provides following features:\n\n* set HTML or data attributes from component assigns\n* set phx-* attributes from component assigns\n* set attributes with any custom prefix such as `@click` or `x-bind:` from\n  [alpinejs](https://github.com/alpinejs/alpine)\n* encode attributes as JSON from an Elixir structure assign\n* validate mandatory attributes\n* set and extend CSS classes from component assigns","ref":"PhxComponentHelpers.html"},{"type":"function","title":"PhxComponentHelpers.extend_class/3","doc":"Provides default css classes and extend them from assigns.\n\nThe class attribute will take provided `default_classes` as a default value and will\nextend them, on a class-by-class basis, with your assigns.\n\nAny CSS class provided in the assigns (by default under the `:class` attribute) will be\nadded to the `default_classes`. You can also remove classes from the `default_classes`\nby using the `!` prefix.\n- `\"!bg-gray-400 bg-blue-200\"` will remove `\"bg-gray-400\"` from default component classes\nand replace it with `\"bg-blue-200\"`\n- `\"!block flex\"` will replace `\"block\"` by `\"flex\"` layout in your component classes.\n- `\"!border* border-2 border-red-400\"` will replace all border classes by\n`\"border-2 border-red-400\"`.","ref":"PhxComponentHelpers.html#extend_class/3"},{"type":"function","title":"Parameters - PhxComponentHelpers.extend_class/3","doc":"* `assigns` - your component assigns\n* `default_classes` - the default classes that will be overridden by your assigns.\nThis parameter can be a binary or a single parameter function that receives all assigns and\nreturns a binary","ref":"PhxComponentHelpers.html#extend_class/3-parameters"},{"type":"function","title":"Options - PhxComponentHelpers.extend_class/3","doc":"* `:attribute` - read & write css classes from & into this key","ref":"PhxComponentHelpers.html#extend_class/3-options"},{"type":"function","title":"Example - PhxComponentHelpers.extend_class/3","doc":"```\nassigns\n|> extend_class(\"bg-blue-500 mt-8\")\n|> extend_class(\"py-4 px-2 divide-y-8 divide-gray-200\", attribute: :wrapper_class)\n|> extend_class(fn assigns ->\n    default = \"p-2 m-4 text-sm \"\n    if assigns[:active], do: default <> \"bg-indigo-500\", else: default <> \"bg-gray-200\"\n   end)\n```\n\n`assigns` now contains `@heex_class` and `@heex_wrapper_class`.\n\nIf your input assigns were `%{class: \"!mt-8 mt-2\", wrapper_class: \"!divide* divide-none\"}` then:\n* `@heex_class` would contain `\"bg-blue-500 mt-2\"`\n* `@heex_wrapper_class` would contain `\"py-4 px-2 divide-none\"`","ref":"PhxComponentHelpers.html#extend_class/3-example"},{"type":"function","title":"PhxComponentHelpers.forward_assigns/2","doc":"Forward and filter assigns to sub components.\nBy default it doesn't forward anything unless you provide it with any combination\nof the options described below.","ref":"PhxComponentHelpers.html#forward_assigns/2"},{"type":"function","title":"Parameters - PhxComponentHelpers.forward_assigns/2","doc":"* `assigns` - your component assigns","ref":"PhxComponentHelpers.html#forward_assigns/2-parameters"},{"type":"function","title":"Options - PhxComponentHelpers.forward_assigns/2","doc":"* `prefix` - will only forward assigns prefixed by the given prefix. Forwarded assign key will no\n   longer have the prefix\n* `take`- is a list of key (without prefix) that will be picked from assigns to be forwarded\n* `merge`- takes a map that will be merged as-is to the output assigns\n\nIf both options are given at the same time, the resulting assigns will be the union of the two.","ref":"PhxComponentHelpers.html#forward_assigns/2-options"},{"type":"function","title":"Example - PhxComponentHelpers.forward_assigns/2","doc":"Following will forward an assign map containing `%{button_id: 42, button_label: \"label\", phx_click: \"save\"}`\nas `%{id: 42, label: \"label\", phx_click: \"save\"}`\n\n```\nforward_assigns(assigns, prefix: :button, take: [:phx_click])\n```","ref":"PhxComponentHelpers.html#forward_assigns/2-example"},{"type":"function","title":"PhxComponentHelpers.has_errors?/1","doc":"If assigns include form and field entries, this function will let you\nknow if the given field is in error or not.\nReturns true or false.","ref":"PhxComponentHelpers.html#has_errors?/1"},{"type":"function","title":"Parameters - PhxComponentHelpers.has_errors?/1","doc":"* `assigns` - your component assigns, which should have `form` and `field` keys.","ref":"PhxComponentHelpers.html#has_errors?/1-parameters"},{"type":"function","title":"PhxComponentHelpers.set_attributes/3","doc":"Extends assigns with heex_* attributes that can be interpolated within your component markup.","ref":"PhxComponentHelpers.html#set_attributes/3"},{"type":"function","title":"Parameters - PhxComponentHelpers.set_attributes/3","doc":"* `assigns` - your component assigns\n  * `attributes` - a list of attributes (atoms) that will be fetched from assigns.\n  Attributes can either be single atoms or tuples in the form `{:atom, default}` to provide\n  default values.","ref":"PhxComponentHelpers.html#set_attributes/3-parameters"},{"type":"function","title":"Options - PhxComponentHelpers.set_attributes/3","doc":"* `:required` - raises if required attributes are absent from assigns\n* `:json` - when true, will JSON encode the assign value\n* `:data` - when true, HTML attributes are prefixed with `data-`\n* `:from` - instead of fetching from all assigns, fetch from one or several :global attributes\n* `:into` - merges all assigns in a single one that can be interpolated at once","ref":"PhxComponentHelpers.html#set_attributes/3-options"},{"type":"function","title":"Example - PhxComponentHelpers.set_attributes/3","doc":"```\nassigns\n|> set_attributes(\n    [:id, :name, label: \"default label\"],\n    required: [:id, :name],\n    into: :attributes\n  )\n|> set_attributes([:value], json: true)\n```\n\n`assigns` now contains :\n- `@heex_id`, `@heex_name`, `@heex_label` and `@heex_value`.\n- `@heex_attributes` which holds the values of `:id`, `:name` and `:label`.","ref":"PhxComponentHelpers.html#set_attributes/3-example"},{"type":"function","title":"PhxComponentHelpers.set_form_attributes/1","doc":"Extends assigns with form related attributes.\n\nIf assigns contain `:form` and `:field` keys then it will set `:id`, `:name`, `:for`,\n`:value`, and `:errors` from received `Phoenix.HTML.Form`.","ref":"PhxComponentHelpers.html#set_form_attributes/1"},{"type":"function","title":"Parameters - PhxComponentHelpers.set_form_attributes/1","doc":"* `assigns` - your component assigns","ref":"PhxComponentHelpers.html#set_form_attributes/1-parameters"},{"type":"function","title":"Example - PhxComponentHelpers.set_form_attributes/1","doc":"```\nassigns\n|> set_form_attributes()\n```","ref":"PhxComponentHelpers.html#set_form_attributes/1-example"},{"type":"function","title":"PhxComponentHelpers.set_phx_attributes/2","doc":"Just a convenient method built on top of `set_prefixed_attributes/3` for phx attributes.\nIt will automatically detect any attribute prefixed by `phx-` from input assigns.\nBy default, the `:into` option of `set_prefixed_attributes/3` is `:phx_attributes`","ref":"PhxComponentHelpers.html#set_phx_attributes/2"},{"type":"function","title":"Example - PhxComponentHelpers.set_phx_attributes/2","doc":"```\nassigns\n|> set_phx_attributes(required: [:\"phx-submit\"], init: [:\"phx-change\"])\n\nassigns\n|> set_phx_attributes(from: :rest)\n```\n\n`assigns` now contains `@heex_phx_change`, `@heex_phx_submit` and `@heex_phx_attributes`.","ref":"PhxComponentHelpers.html#set_phx_attributes/2-example"},{"type":"function","title":"PhxComponentHelpers.set_prefixed_attributes/3","doc":"Extends assigns with prefixed attributes that can be interpolated within\nyour component markup. It will automatically detect any attribute prefixed by\nany of the given prefixes from input assigns.\n\nCan be used for intance to easily map `alpinejs` html attributes.","ref":"PhxComponentHelpers.html#set_prefixed_attributes/3"},{"type":"function","title":"Parameters - PhxComponentHelpers.set_prefixed_attributes/3","doc":"* `assigns` - your component assigns\n* `prefixes` - a list of prefix as binaries","ref":"PhxComponentHelpers.html#set_prefixed_attributes/3-parameters"},{"type":"function","title":"Options - PhxComponentHelpers.set_prefixed_attributes/3","doc":"* `:init` - a list of attributes that will be initialized if absent from assigns\n* `:required` - raises if required attributes are absent from assigns\n* `:from` - instead of fetching from all assigns, fetch from one or several :global attributes\n* `:into` - merges all assigns in a single one that can be interpolated at once","ref":"PhxComponentHelpers.html#set_prefixed_attributes/3-options"},{"type":"function","title":"Example - PhxComponentHelpers.set_prefixed_attributes/3","doc":"```\nassigns\n|> set_prefixed_attributes(\n  [\"@click\", \"x-bind:\"],\n  required: [\"x-bind:class\"],\n  into: :alpine_attributes\n)\n|> set_prefixed_attributes(\n  [\"aria-\"],\n  from: :rest,\n  into: :aria_attributes\n)\n```\n\n`assigns` now contains `@heex_click`, `@heex_x-bind:class`\nand `@heex_alpine_attributes`.","ref":"PhxComponentHelpers.html#set_prefixed_attributes/3-example"},{"type":"function","title":"PhxComponentHelpers.validate_required_attributes/2","doc":"Validates that attributes are present in assigns.\nRaises an `ArgumentError` if any attribute is missing.","ref":"PhxComponentHelpers.html#validate_required_attributes/2"},{"type":"function","title":"Example - PhxComponentHelpers.validate_required_attributes/2","doc":"```\nassigns\n|> validate_required_attributes([:id, :label])\n```","ref":"PhxComponentHelpers.html#validate_required_attributes/2-example"},{"type":"module","title":"PhxViewHelpers","doc":"`PhxComponentHelpers` are helper functions meant to be used within Phoenix your views to\n facilitate usage of live_components inside templates.","ref":"PhxViewHelpers.html"},{"type":"function","title":"PhxViewHelpers.extend_form_class/2","doc":"Extends `Phoenix.Component.form/1` options to merge css class as with\n`PhxComponentHelpers.extend_class/2`.\n\nIt's useful to define your own `my_form` function with default css classes that still can be\noverriden from the template.","ref":"PhxViewHelpers.html#extend_form_class/2"},{"type":"function","title":"Example - PhxViewHelpers.extend_form_class/2","doc":"```\ndef my_form(options) do\n  new_options = extend_form_class(options, \"mt-4 space-y-2\")\n  Component.form(new_options)\nend\n```","ref":"PhxViewHelpers.html#extend_form_class/2-example"},{"type":"extras","title":"PhxComponentHelpers","doc":"# PhxComponentHelpers\n\n[![github](https://github.com/cblavier/phx_component_helpers/actions/workflows/elixir.yml/badge.svg)](https://github.com/cblavier/phx_component_helpers/actions/workflows/elixir.yml)\n[![codecov](https://codecov.io/gh/cblavier/phx_component_helpers/branch/main/graph/badge.svg)](https://codecov.io/gh/cblavier/phx_component_helpers)\n[![Hex pm](http://img.shields.io/hexpm/v/phx_component_helpers.svg?style=flat)](https://hex.pm/packages/phx_component_helpers)\n\n👉 [Demonstration & Code samples](https://phx-component-helpers-demo.onrender.com)","ref":"readme.html"},{"type":"extras","title":"Presentation - PhxComponentHelpers","doc":"`PhxComponentHelpers` provides helper functions meant to be used within Phoenix LiveView to make your components more configurable and extensible from templates.\n\nIt provides the following features:\n\n- set HTML, data or phx attributes from component assigns\n- set a bunch of attributes at once with any custom prefix such as `@click` or `x-bind:` (for [alpinejs](https://github.com/alpinejs/alpine) users)\n- validate mandatory attributes\n- set and extend CSS classes from component assigns\n- forward a subset of assigns to child components","ref":"readme.html#presentation"},{"type":"extras","title":"Motivation - PhxComponentHelpers","doc":"Writing a library of stateless components is a great way to improve consistency in both your UI and code and also to get a significant productivity boost.\n\nThe best components can be used _as-is_ without any further configuration, but are versatile enough to be customized from templates or higher level components.\n\nWriting such components is not difficult, but involves a lot of boilerplate code. `PhxComponentHelpers` is here to alleviate the pain.","ref":"readme.html#motivation"},{"type":"extras","title":"Example - PhxComponentHelpers","doc":"A lot of code samples are available [on this site](https://phx-component-helpers-demo.onrender.com), but basically `PhxComponentHelpers` allows you to write components as such:\n\n```elixir\ndefmodule Forms.Button do\n  use Phoenix.Component\n  import PhxComponentHelpers\n\n  def button(assigns) do\n    assigns =\n      assigns\n      |> extend_class(\"bg-blue-700 hover:bg-blue-900 ...\")\n      |> set_attributes([:type, :id, :label], required: [:id])\n      |> set_phx_attributes()\n\n    ~H\"\"\"\n     \n      <%= @label %>\n     \n    \"\"\"\n  end\nend\n```\n\nFrom templates, it looks like this:\n\n```heex\n<.form id=\"form\" phx-submit=\"form_submit\" class=\"divide-none\">\n\n  <.input_group>\n    <.label for=\"name\" label=\"Name\"/>\n    <.text_input name=\"name\" value={@my.name}/>\n  </.input_group>\n\n  <.button_group class=\"pt-2\">\n    <.button type=\"submit\" phx-click=\"btn-click\" label=\"Save\"/>\n  </.button_group>\n\n</.form>\n```","ref":"readme.html#example"},{"type":"extras","title":"How does it play with the PETAL stack? - PhxComponentHelpers","doc":"[PETAL](https://thinkingelixir.com/petal-stack-in-elixir/) stands for Phoenix - Elixir - TailwindCSS - Alpine.js - LiveView. In recent months it has become quite popular in the Elixir ecosystem and `PhxComponentHelpers` is meant to fit in.\n\n- [TailwindCSS](https://tailwindcss.com) provides a new way to structure CSS, but keeping good HTML hygiene requires to rely on a component-oriented library.\n- [Alpine.js](https://github.com/alpinejs/alpine) is the Javascript counterpart of Tailwind. It lets you define dynamic behaviour right from your templates using HTML attributes.\n\nThe point of developing good components is to provide strong defaults in the component so that they can be used _as_-is, but also to let these defaults be overridden right from the templates.\n\nHere is the definition of a typical Form button, with `Tailwind` & `Alpine`:\n\n```elixir\ndefmodule Forms.Button do\n  use Phoenix.Component\n  import PhxComponentHelpers\n\n  @css_class \"inline-flex items-center justify-center p-3 w-5 h-5 border \\\n              border-transparent text-2xl leading-4 font-medium rounded-md \\\n              text-white bg-primary hover:bg-primary-hover\"\n\n  def button(assigns) do\n    assigns =\n      assigns\n      |> extend_class(@css_class)\n      |> set_phx_attributes()\n      |> set_prefixed_attributes([\"@click\", \"x-bind:\"],\n        into: :alpine_attributes,\n        required: [\"@click\"]\n      )\n\n    ~H\"\"\"\n     \n      <%= render_block(@inner_block) %>\n     \n    \"\"\"\n  end\nend\n```\n\nThen in your `html.heex` template you can imagine the following code, providing `@click` behaviour and overriding just the few tailwind css classes you need (only `p-*`, `w-*` and `h-*` will be replaced). No `phx` behaviour here, but it's ok, it won't break ;-)\n\n```elixir\n<.button class=\"!p-* p-0 !w-* w-7 !h-* h-7\" \"@click\"=\"$dispatch('closeslideover')\">\n  <.icon icon={:plus_circle}/>\n</.button>\n```","ref":"readme.html#how-does-it-play-with-the-petal-stack"},{"type":"extras","title":"Forms - PhxComponentHelpers","doc":"This library also provides `Phoenix.HTML.Form` related functions so you can easily write your own `my_form_for` function with your css defaults.\n\n```elixir\ndef my_form_for(options) when is_list(options) do\n  options\n  |> extend_form_class(\"mt-4 space-y-2\")\n  |> Phoenix.LiveView.Helpers.form()\nend\n```\n\nThen you only need to use `PhxComponentHelpers.set_form_attributes/1` within your own form components in order to fetch names & values from the form. Your template will then look like this:\n\n```heex\n<.my_form_for let={f} for={@changeset} phx-submit=\"form_submit\" class=\"divide-none\">\n  <.input_group>\n    <.label form={f} field={:name} label=\"Name\"/>\n    <.text_input form={f} field={:name}/>\n  </.input_group>\n\n  <.button_group class=\"pt-2\">\n    <.button type=\"submit\" label=\"Save\"/>\n  </.button_group>\n</.my_form_for>\n```","ref":"readme.html#forms"},{"type":"extras","title":"Compared to Surface - PhxComponentHelpers","doc":"[Surface](https://github.com/surface-ui/surface) is a library built on top of Phoenix LiveView. Surface is much more ambitious and complex than `PhxComponentHelpers` (which obviously isn't a framework, just helpers ...).\n\n`Surface` really changes the way you code user interfaces and components (you almost won't be using HTML templates anymore) whereas `PhxComponentHelpers` is just some syntactic sugar to help you use raw `phoenix_live_view`.","ref":"readme.html#compared-to-surface"},{"type":"extras","title":"Documentation - PhxComponentHelpers","doc":"Available on [https://hexdocs.pm](https://hexdocs.pm/phx_component_helpers)","ref":"readme.html#documentation"},{"type":"extras","title":"Installation - PhxComponentHelpers","doc":"Add the following to your `mix.exs`.\n\n```elixir\ndef deps do\n  [\n    {:phx_component_helpers, \"~> 1.4.0\"},\n    {:jason, \"~> 1.0\"} # only required if you want to use json encoding options\n  ]\nend\n```","ref":"readme.html#installation"}],"content_type":"text/markdown"}